@@ -1,4 +1,4 @@
-# $Id: Macro.pm,v 1.5 2008/07/13 19:36:52 T0071173 Exp $
+# $Id: Macro.pm,v 1.8 2008/12/29 09:52:14 Paulo Custodio Exp $
 
 # Implement the z80masm macro processor
 
@@ -10,7 +10,7 @@ use 5.008;
 
 use HOP::Stream ':all';
 
-our $VERSION = '2.01';
+our $VERSION = '2.02';
 
 use vars qw(@EXPORT);
 use base qw(Exporter);
@@ -39,7 +39,7 @@ sub _define_macro {
 	# get macro name
 	my $token;
 	($token = head($input)) && $token->[0] eq 'NAME'
-		or die "Error defining macro: name needed\n";
+		or die "Macro name needed\n";
 	$macro->name( $token->[1] );
 	drop($input);
 	
@@ -54,10 +54,18 @@ sub _define_macro {
 
 	# skip {
 	my $opened_brace;
-	if (($token = head($input)) && $token->[0] eq '{') {
+	($token = head($input)) 
+		or die "Macro body needed\n";	
+	if ($token->[0] eq '{') {
 		drop($input);
 		$opened_brace++;
 	}
+	elsif (exists($STMT_END{$token->[0]})) {
+		# OK, macro body follows on next line
+	}
+	else {
+		die "Unexpected '",$token->[0],"'\n";
+	}
 		
 	# retrieve tokens
 	my @macro_tokens;
@@ -106,6 +114,7 @@ sub _define_macro {
 		}
 		$last_stmt_end = exists($STMT_END{$token->[0]});
 	}
+	die "Macro body not finished\n" unless $token;
 	die "Unmatched braces\n" if $parens != 0;
 	
 	# macro contents and line tokens as streams
